{% extends "vendor/base.html" %}
{% load i18n %}
{% load crispy_forms_tags %}

{% block vendor_content %}
    <div class="container">
        <div class="row">
            <div class="col-lg container-fluid page__container">
                
                <form method="post">{% csrf_token %}
                    {{ form|crispy }}
                    <button class="btn btn-danger">{% trans 'Request Refund' %}</button>
                </form>
            </div>
        </div>
    </div>
{% endblock %}