{% extends "vendor/manage/base.html" %}
{% load i18n %}
{% block vendor_content %}


<nav aria-label="breadcrumb">
    <h1>{% trans 'Product' %}</h1>
</nav>

<div class='row'>
    <div class='d-flex flex-column mr-auto ml-auto w-75'>
        <div class='card'>
            <div class='card-header'>
                {% trans 'Product' %}
            </div>
            <div class='card-body'>
                <form method="post">
                    {% csrf_token %}
                    {{ form.media }}
                    {{ form.as_p }}
                    <button class="btn btn-primary btn-lg btn-block" type="submit">{% trans 'Save Product' %}</button>
                </form>
            </div>
        </div>
    </div>
</div>
    
    
{% endblock %}